@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
+# Copyright (C) 2006, 2007, 2008, 2009 Loic Dachary <loic@dachary.org>
 # Copyright (C)       2008, 2009 Bradley M. Kuhn <bkuhn@ebb.org>
 # Copyright (C) 2004, 2005, 2006 Mekensleep <licensing@mekensleep.com>
 #                                24 rue vieille du temple, 75004 Paris
@@ -56,7 +56,7 @@ def equalizeCandidates(games):
         count = game.allCount()
         if count < threshold:
             want_players.append([ game.id, game.max_players - count ])
-        elif not game.isRunning():
+        elif game.isEndOrNull():
             serials = game.serialsAllSorted()
             provide_players.append((game.id, serials[:count - threshold]))
     return ( want_players, provide_players )
@@ -107,14 +107,14 @@ def breakGames(games, verbose = 0, log_message = None):
     # Games not running first, then games running.
     # Each is sorted with games that have least players first.
     #
-    games.sort(lambda a,b: a.isRunning() - b.isRunning() or int(a.allCount() - b.allCount()) )
+    games.sort(lambda a,b: b.isEndOrNull() - a.isEndOrNull() or int(a.allCount() - b.allCount()) )
 
     to_break = [ {
         "id": game.id,
         "seats_left": game.max_players - game.allCount(),
         "serials": game.serialsAll(),
         "to_add": [],
-        "running": game.isRunning() } for game in games ]
+        "running": not game.isEndOrNull() } for game in games ]
 
     if verbose > 2: log_message("balanceGames breakGames: %s" % to_break)
     results = []
@@ -304,7 +304,12 @@ class PokerTournament:
                 self.changeState(TOURNAMENT_STATE_BREAK_WAIT)
                 
         if self.state == TOURNAMENT_STATE_BREAK_WAIT:
-            self.breaks_games_id.append(game_id)
+            #
+            # game_id is 0 when updateBreak is called after a table was destroyed
+            # as a side effect of balanceGames
+            #
+            if game_id > 0:
+                self.breaks_games_id.append(game_id)
             on_break = True
             for game in self.games:
                 if game.id not in self.breaks_games_id:
@@ -490,9 +495,15 @@ class PokerTournament:
             self.changeState(TOURNAMENT_STATE_COMPLETE)
             return False
         else:
-            if loosers_count > 0 or self.need_balance: self.balanceGames()
+            if loosers_count > 0 or self.need_balance:
+                self.balanceGames()
             if self.id2game.has_key(game_id):
                 self.updateBreak(game_id)
+            else:
+                #
+                # This happens if game_id was destroyed by the call to balanceGames above
+                #
+                self.updateBreak(0)
             return True
         
     def balanceGames(self):
@@ -504,7 +515,10 @@ class PokerTournament:
         for (from_id, to_id, serials) in to_break:
             for serial in serials:
                 if self.verbose > 2: self.message("balanceGames: player %d moved from %d to %d" % ( serial, from_id, to_id ))
-                self.callback_move_player(self, from_id, to_id, serial)
+                if self.state == TOURNAMENT_STATE_REGISTERING:
+                    self.movePlayer(from_id, to_id, serial)
+                else:
+                    self.callback_move_player(self, from_id, to_id, serial)
             games_broken[from_id] = True
 
         if len(to_break) > 0:
@@ -519,7 +533,10 @@ class PokerTournament:
         to_equalize = equalizeGames(self.games, self.verbose, self.message)
         for (from_id, to_id, serial) in to_equalize:
             if self.verbose > 2: self.message("balanceGames: player %d moved from %d to %d" % ( serial, from_id, to_id ))
-            self.callback_move_player(self, from_id, to_id, serial)
+            if self.state == TOURNAMENT_STATE_REGISTERING:
+                self.movePlayer(from_id, to_id, serial)
+            else:
+                self.callback_move_player(self, from_id, to_id, serial)
 
         ( want_players, provide_players ) = equalizeCandidates(self.games)
         self.need_balance = want_players and not provide_players
